﻿using System;
using System.Collections.Generic;
//using System.Drawing;
using System.Linq;
using System.Media;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using static System.Console;
using static Охота_на_Cкалоедов.Properties.Resources;


// Игра Охота на Скалоедов

namespace Охота_на_Cкалоедов
{
    public static class GDIEX
    {
        public static BitmapSource ToBitmapSource(this System.Drawing.Bitmap bitmap)
        {
            System.IO.MemoryStream stream = new System.IO.MemoryStream();
            using (stream)
            {
                bitmap.Save(stream, System.Drawing.Imaging.ImageFormat.Png);// .Bmp);
                stream.Position = 0;

                var result = BitmapFrame.Create(stream, BitmapCreateOptions.None, BitmapCacheOption.OnLoad);
                result.Freeze();
                return result;
            }
        }
    }


    public partial class MainWindow : Window
    {
        // макс. число Скалоедов:
        const int MAX_SKALOEDY = 32;

        // число Скалоедов в текущей игре:
        int nSkaloedy = MAX_SKALOEDY;

        // пустая клетка:
        public const int EMPTY = -1;

        // осталось Скалоедов:
        int ostSkaloedy;

        // список всех Скалоедов:
        List<Skaloed> skaloedy = new List<Skaloed>();




        // размеры клеток (картинок) в пикселях:
        public const int CELL_WIDTH = 62,
            CELL_HEIGHT = 62;

        // названия файлов с картинками:
        readonly System.Drawing.Bitmap[] imageNames = {skala_,
           hod_,
           ohotnik_,
           skaloed_,
        };
        // список картинок:
        public static List<Image> images = new List<Image>();

        // картинки с поздравлениями:
        Image imgPozdrOhotnik,
            imgPozdrSkaloedy;

        // проигрыватели звука:
        SoundPlayer spStepo = new SoundPlayer();
        SoundPlayer spSteps = new SoundPlayer();
        SoundPlayer spWino = new SoundPlayer();
        SoundPlayer spWins = new SoundPlayer();

        // непроходимые и прoходимые клетки:
        public const int SKALA = 0, HOD = 1,
        // игроки:
        OHOTNIK = 2,
        SKALOED = 3;

        // число клеток по горизонтали 
        // и вертикали:
        const int FIELD_WIDTH = 10, FIELD_HEIGHT = 10;

        // лабиринт:    
        Cell[,] labyrinth = new Cell[FIELD_WIDTH, FIELD_HEIGHT];

        // Скалоед:
        //Skaloed skaloed;
        // Охотник:
        Ohotnik ohotnik;

        //// координаты Скалоеда:
        //Point skaloedCoods = new Point(0, 0);
        //// координаты Охотника:
        //Point ohotnikCoords = new Point(0, 0);

        // счёт игры:
        int[] result = { 0, 0, 0, 0, 0 };
        // победитель:
        int winner = 0;
        // число ходов:
        int nMove = 0;
        // задержка хода Скалоеда:
        //int pause = 400;
        // статус игры:
        int status = 0;
        // флаг окончания игры:
        bool gameOver = false;

        Random rand = new Random();


        public MainWindow()
        {
            InitializeComponent();
        }


        // ЗАГРУЖАЕМ МЕДИАФАЙЛЫ
        private void MainWindow1_Loaded(object sender, RoutedEventArgs e)
        {
            // загружаем в программу
            // звуковые файлы:
            spWino.Stream = wino;
            spWins.Stream = wins;
            spStepo.Stream = stepo;
            spSteps.Stream = steps;

            // загружаем в программу
            // картинки:
            foreach (var imgn in imageNames)
            {
                var image = new Image
                {
                    Source = imgn.ToBitmapSource()
                };
                images.Add(image);
            }

            // загружаем картинки с поздравлениями:
            imgPozdrOhotnik = new Image
            {
                Source = pozdrOhotnik_.ToBitmapSource()
            };
            // добавляем к канве:
            canvas.Children.Add(imgPozdrOhotnik);
            // размеры картинки:
            var w = imgPozdrOhotnik.Source.Width;
            var h = imgPozdrOhotnik.Source.Height;
            // центрируем на канве:
            Canvas.SetTop(imgPozdrOhotnik, (canvas.Height - h) / 2);
            Canvas.SetLeft(imgPozdrOhotnik, (canvas.Width - w) / 2);

            imgPozdrSkaloedy = new Image
            {
                Source = pozdrSkaloedy_.ToBitmapSource()
            };
            // добавляем к канве:
            canvas.Children.Add(imgPozdrSkaloedy);
            // размеры картинки:
            w = imgPozdrSkaloedy.Source.Width;
            h = imgPozdrSkaloedy.Source.Height;
            // центрируем на канве:
            Canvas.SetTop(imgPozdrSkaloedy, (canvas.Height - h) / 2);
            Canvas.SetLeft(imgPozdrSkaloedy, (canvas.Width - w) / 2);

            // создаём игровое поле:
            CreateField();
            NewGame();
        }


        // СОЗДАЁМ ИГРОВОЕ ПОЛЕ
        private void CreateField()
        {
            // создаём игровое поле-лабиринт:
            for (int row = 0; row < FIELD_HEIGHT; row++)
                for (int col = 0; col < FIELD_WIDTH; col++)
                {
                    // создаём клетку:
                    labyrinth[col, row] = new Cell(this.canvas, col, row)
                    {
                        Cursor = Cursors.Hand
                    };
                    // добавляем метод-обработчмк
                    // нажатия на картинку:
                    labyrinth[col, row].MouseLeftButtonDown += Cell_Click;
                }
        }


        // НАЧИНАЕМ НОВУЮ ИГРУ
        private void ButtonNewGame_Click(object sender, RoutedEventArgs e)
        {
            NewGame();
        }

        // НОВАЯ ИГРА
        void NewGame()
        {
            // сначала все клетки проходимые:
            for (var row = 0; row < FIELD_HEIGHT; ++row)
                for (var col = 0; col < FIELD_WIDTH; ++col)
                {
                    labyrinth[col, row].Item = HOD;
                    labyrinth[col, row].Who = EMPTY;
                }

            // скальная порода:
            for (var j = 1; j < FIELD_HEIGHT; ++j)
                for (var i = 0; i < FIELD_HEIGHT; ++i)
                {
                    // 
                    if (rand.Next(100) > 50)
                    {
                        labyrinth[i, j].Item = SKALA;
                    }
                }

            // создаём Охотника:
            canvas.Children.Remove(ohotnik);
            ohotnik = new Ohotnik(canvas)
            {
                Cursor = Cursors.Hand
            };
            ohotnik.MouseLeftButtonDown += Ohotnik_Click;
            // он стоит в левом верхнем углу?
            var cl = 0;
            var rw = 0;
            ohotnik.Col = cl;
            ohotnik.Row = rw;
            // в проходимой клетке:
            labyrinth[cl, rw].Item = HOD;
            labyrinth[cl, rw].Who = OHOTNIK;


            // создаём Скалоедов:
            Skaloed skaloed = null;
            if (skaloedy.Count != 0)
            {
                for (int i = 0; i < skaloedy.Count; ++i)
                {
                    skaloed = skaloedy[i];
                    canvas.Children.Remove(skaloed);
                }
                skaloedy.Clear();
            }

            var nsk = nSkaloedy;
            while (nsk > 0)
            {
                do
                {
                    // координаты Скалоеда:
                    cl = rand.Next(FIELD_WIDTH);
                    rw = rand.Next(FIELD_HEIGHT);
                } while (labyrinth[cl, rw].Who != EMPTY);
                labyrinth[cl, rw].Who = SKALOED;
                //skaloed = new Skaloed(canvas, cl, rw, nsk * 50)
                skaloed = new Skaloed(canvas, cl, rw, 250)
                {
                    Cursor = Cursors.Hand
                };
                // добавляем метод-обработчмк
                // нажатия на картинку:
                skaloed.MouseLeftButtonDown += Skaloed_Click;
                skaloedy.Add(skaloed);
                --nsk;
            }

            // осталось Скалоедов:
            ostSkaloedy = nSkaloedy;

            // таблички с поздравлениями:
            canvas.Children.Remove(imgPozdrSkaloedy);
            canvas.Children.Add(imgPozdrSkaloedy);
            imgPozdrSkaloedy.Visibility = Visibility.Hidden;

            canvas.Children.Remove(imgPozdrOhotnik);
            canvas.Children.Add(imgPozdrOhotnik);
            imgPozdrOhotnik.Visibility = Visibility.Hidden;

            // обнуляем число ходов:
            this.nMove = 0;
            // победитель:
            this.winner = 0;
            // ход Охотника:
            this.status = OHOTNIK;
            // печатаем текущую информацию:
            PrintInfo();
            // игра началась:
            gameOver = false;
        }


        // ПЕЧАТАЕМ ИНФОРМАЦИЮ
        void PrintInfo()
        {
            LabelMove.Content = "Ход: " + nMove;
            LabelResultOhotnik.Content = "Охотник    " + result[OHOTNIK];
            LabelResultSkaloed.Content = "Скалоеды " + result[SKALOED];
            LabelNSkaloedy.Content = ostSkaloedy;
        }


        // ЩЁЛКАЕМ ПО СКАЛОЕДУ
        private void Skaloed_Click(object sender, MouseButtonEventArgs e)
        {
            // игра закончена:
            if (gameOver)
                return;

            // сейчас ход Скалоедов:
            if (status != OHOTNIK)
                return;

            // клмкнутый Скалоед:
            var skaloed = (Skaloed)sender;

            // текущая клетка со Скалоедом:
            var cols = skaloed.Col;
            var rows = skaloed.Row;

            // текущая клетка с Охотником:
            var colo = ohotnik.Col;
            var rowo = ohotnik.Row;

            var dc = colo - cols;
            var dr = rowo - rows;

            // расстояние от кликнутой клетки:
            var dcol = Math.Abs(dc);
            var drow = Math.Abs(dr);

            // клетка должнв быть рядом с Охотником:
            if (dcol + drow != 1)
                return;

            // убираем Охотника из клетки:
            labyrinth[colo, rowo].Who = EMPTY;
            // переход Охотника:
            ohotnik.Animation(-dc, -dr);

            // Скалоед крякнул:
            spWino.Play();

            // удаляем Скалоеда:
            RemoveSkaloed(skaloed);
        }



        // УДАЛЯЕМ СКАЛОЕДА
        async void RemoveSkaloed(Skaloed skaloed)
        {

            await Task.Delay(250);
            // увеличиваем счётчик ходов:
            ++this.nMove;
            // уменьшаем число Скалоедов:
            --ostSkaloedy;
            canvas.Children.Remove(skaloed);
            // текущая клетка со Скалоедом:
            var cols = skaloed.Col;
            var rows = skaloed.Row;
            labyrinth[cols, rows].Item = HOD;
            labyrinth[cols, rows].Who = OHOTNIK;

            skaloedy.Remove(skaloed);
            // обновляем информацию:
            PrintInfo();
            if (ostSkaloedy == 0)
            {
                // Охотник победил:
                winner = OHOTNIK;
                GameOver();
            }
            else
            {
                status = SKALOED;
                MoveSkaloedy();
            }
        }


        // ЩЁЛКАЕМ ПО ОХОТНИКУ
        private void Ohotnik_Click(object sender, MouseButtonEventArgs e)
        {
            // игра закончена:
            if (gameOver)
                return;

            // сейчас ход Скалоеда:
            if (status != OHOTNIK)
                return;

            ++nMove;
            PrintInfo();

            // Охотник пропускает ход.
            // Ход переходит к Скалоеду:
            status = SKALOED;
            MoveSkaloedy();
        }


        // ХОД СКАЛОЕДОВ
        void MoveSkaloedy()
        {
            // игра закончена:
            if (gameOver)
                return;

            spSteps.Play();
            // по всем Скалоедам:
            for (int i = 0; i < skaloedy.Count; ++i)
            {
                // очередной Скалоед:
                var skaloed = skaloedy[i];
                // клетка Скалоеда:
                var col = skaloed.Col;
                var row = skaloed.Row;

                // возможные ходы Скалоеда:
                var smoves = String.Empty;
                if (TestSkaloed(skaloed, 'L'))
                    smoves += "L";
                if (TestSkaloed(skaloed, 'R'))
                    smoves += "R";
                if (TestSkaloed(skaloed, 'U'))
                    smoves += "U";
                if (TestSkaloed(skaloed, 'D'))
                    smoves += "D";

                // если у Скалоеда нет ходов:
                if (smoves.Length == 0)
                    continue;

                // выбираем случайный ход:
                var n = rand.Next(smoves.Length);

                // Скалоед делает ход:
                var dir = smoves[n];
                int dc = 0;
                int dr = 0;
                switch (dir)
                {
                    case 'L':
                        dc -= 1;
                        break;
                    case 'R':
                        dc += 1;
                        break;
                    case 'U':
                        dr -= 1;
                        break;
                    case 'D':
                        dr += 1;
                        break;
                }
                // новая клетка со Скалоедом:
                var cols = col + dc;
                var rows = row + dr;

                labyrinth[col, row].Who = EMPTY;
                labyrinth[cols, rows].Who = SKALOED;

                // если под ним скала:
                if (labyrinth[col, row].Item == SKALA)
                    // он делает проход:
                    labyrinth[col, row].Item = HOD;
                else
                    // иначе засыпает проход:
                    labyrinth[col, row].Item = SKALA;

                skaloed.Animation(dc, dr);
                // новая клетка со Скалоедом:
                cols = skaloed.Col;
                rows = skaloed.Row;

                // клетка с Охотником:
                var colo = ohotnik.Col;
                var rowo = ohotnik.Row;

                // Скалоед победил:
                if (cols == colo && rows == rowo)
                {
                    winner = SKALOED;
                    GameOver();
                    return;
                }
            } // перебираем Скалоедов

            // ход Охотника:
            status = OHOTNIK;
        }


        // ИГРА ЗАКОНЧЕНА
        async void GameOver()
        {
            if (gameOver)
                return;

            gameOver = true;

            // небольшая задержка:
            await Task.Delay(500);

            if (winner == OHOTNIK)
            {
                imgPozdrOhotnik.Visibility = Visibility.Visible;
            }
            else
            {
                spWins.Play();
                ohotnik.Visibility = Visibility.Hidden;
                imgPozdrSkaloedy.Visibility = Visibility.Visible;
            }

            result[winner]++;
            PrintInfo();
        }


        // ПРОВЕРЯЕМ ХОД СКАЛОЕДА
        bool TestSkaloed(Skaloed skaloed, char dir)
        {
            var x = 0;
            var y = 0;
            // клетка Скалоеда:
            var col = skaloed.Col;
            var row = skaloed.Row;
            // ход влево:
            if (dir == 'L')
            {
                // новые координаты Скалоеда:
                x = col - 1;
                y = row;
            }
            // ход вправо:
            if (dir == 'R')
            {
                x = col + 1;
                y = row;
            }
            // ход вверх:
            if (dir == 'U')
            {
                x = col;
                y = row - 1;
            }
            // ход вниз:
            if (dir == 'D')
            {
                x = col;
                y = row + 1;
            }

            // выход за граница лабиринта:
            if (x < 0 || x >= FIELD_WIDTH ||
               y < 0 || y >= FIELD_HEIGHT)
                return false;

            // в клетке находится другой Скалоед:
            if (labyrinth[x, y].Who == SKALOED)
                return false;

            return true;
        }


        // ЩЁЛКАЕМ ПО КЛЕТКЕ
        private void Cell_Click(object sender, MouseButtonEventArgs e)
        {
            // игра закончена:
            if (gameOver)
                return;

            // кликнутая клетка:
            var cell = (Cell)sender;

            // передвигаем Охотника:
            MoveOhotnik(cell.Col, cell.Row);
        }


        // ПЕРЕХОД ОХОТНИКА НА ЗАДАННУЮ КЛЕТКУ
        private void MoveOhotnik(int colm, int rowm)
        {
            // colm, rowm - координаты кликнутой клетки 

            // сейчас ход Скалоеда:
            if (status != OHOTNIK)
                return;

            // текущая клетка с Охотником:
            var colo = ohotnik.Col;
            var rowo = ohotnik.Row;

            // разность координат:
            var dc = colo - colm;
            var dr = rowo - rowm;

            // расстояние от кликнутой клетки:
            var dcol = Math.Abs(dc);
            var drow = Math.Abs(dr);

            // клетка должнв быть рядом с Охотником:
            if (dcol + drow != 1)
                return;

            // нельзя пойти в непроходимую клетку:
            if (labyrinth[colm, rowm].Item == SKALA)
                return;

            // анимируем переход:
            ohotnik.Animation(-dc, -dr);
            // звук шагов:
            spStepo.Play();

            // увеличиваем счётчик ходов:
            ++this.nMove;
            // обновляем информацию на экране:
            PrintInfo();

            status = SKALOED;
            MoveSkaloedy();
        }
    }
}
