﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Media;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;

using static System.Console;
using static Бокалы.Properties.Resources;


namespace Бокалы
{
    public static class GDIEX
    {
        public static BitmapSource ToBitmapSource(this System.Drawing.Bitmap bitmap)
        {
            System.IO.MemoryStream stream = new System.IO.MemoryStream();
            using (stream)
            {
                bitmap.Save(stream, System.Drawing.Imaging.ImageFormat.Png);// .Bmp);
                stream.Position = 0;

                var result = BitmapFrame.Create(stream, BitmapCreateOptions.None, BitmapCacheOption.OnLoad);
                result.Freeze();
                return result;
            }
        }
    }


    public partial class MainWindow : Window
    {
        // проигрыватели звука:
        SoundPlayer spChose = new SoundPlayer();
        SoundPlayer spError = new SoundPlayer();
        SoundPlayer spWin = new SoundPlayer();

        // сообщение:
        String message = String.Empty;

        // картинки с бокалами:
        Image imgBlue,
            //imgDownBlue,
            imgRed;
        //imgDownRed;

        // картинка с поздавлением:
        Image imgPozdrBig;

        // мин. макс. число бокалов:
        int MIN_BOKALY = 7;
        int MAX_BOKALY = 12;

        // размеры картинок:
        public const int IMAGE_WIDTH = 61,
            IMAGE_HEIGHT = 101;

        // список бокалов:
        List<Bokal> bokaly = new List<Bokal>();

        // число бокалов:
        int nBok = 0;
        // число бокалов, которые нужно
        // перевернуть за 1 раз:
        public int nPerev = 0;
        // осталось неперевёрнутых бокалов:
        int restBok = 0;
        // число выбранных бокалов:
        public int nChose = 0;

        // состояние игры:
        bool gameOver = true;
        // число ходов:
        int nMove = 0;

        // ГПСЧ:
        Random rand = new Random();


        public MainWindow()
        {
            InitializeComponent();
        }


        // ЗАГРУЖАЕМ МЕДИАФАЙЛЫ
        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            // загружаем в программу
            // звуковые файлы:
            spWin.Stream = win;
            spChose.Stream = chose;
            spError.Stream = error;

            // картинка с поздравлением:
            imgPozdrBig = new Image
            {
                Source = pozdrBig.ToBitmapSource()
            };
            // размеры картинки:
            var w = pozdrBig.Width;
            var h = pozdrBig.Height;
            // центрируем на канве:
            Canvas.SetTop(imgPozdrBig, (canvas.Height - h) / 2);
            Canvas.SetLeft(imgPozdrBig, (canvas.Width - w) / 2);

            // бокалы:
            imgBlue = new Image
            {
                Source = upblue.ToBitmapSource()
            };
            imgRed = new Image
            {
                Source = upred.ToBitmapSource()
            };

            // начинаем игру:
            NewGame();
        }


        // НАЖИМАЕМ КНОПКУ "НОВАЯ ИГРА"
        private void ButtonNewGame_Click(object sender, RoutedEventArgs e)
        {
            NewGame();
        }

        // НАЧИНАЕМ НОВУЮ ИГРУ
        void NewGame()
        {
            // скрываем табличку:
            imgPozdrBig.Visibility = Visibility.Hidden;

            // загадываем число бокалов:
            nBok = rand.Next(MIN_BOKALY, MAX_BOKALY + 1);

            // загадываем число бокалов, 
            // которые нужно перевернуть за 1 раз:
            nPerev = rand.Next(2, 6);

            // если число бокалов нечётное, а переворачиваем чётное
            // число бокалов, то задача неразрешима.
            // Добавляем ещё 1 бокал:
            if ((nBok % 2 == 1) && (nPerev % 2 == 0))
                ++nPerev;

            // осталось перевернуть бокалов:
            restBok = nBok;

            bokaly.Clear();
            canvas.Children.Clear();
            // создаём бокалы:
            for (int i = 0; i < nBok; ++i)
            {
                var match = new Bokal(canvas, i * 65 + 20, 195)
                {
                    Source = imgBlue.Source
                };
                // добавляем метод-обработчмк
                // нажатия на картинку:
                match.MouseLeftButtonDown += Bokal_Click;
                bokaly.Add(match);
            }

            // добавляем на канву табличку:
            canvas.Children.Add(imgPozdrBig);

            // условия игры:
            var pravila = "Число бокалов равно " + nBok +
                          ". За один раз нужно перевернуть " + nPerev + ".";
            LabelPravila.Content = pravila;

            // печатаем подсказку:
            int m = nBok;
            int n = nPerev;
            int k = 0;
            while ((n * k < m) || ((n * k - m) % 2 != 0))
                ++k;
            var hint = "Число ходов = " + k + ".  Число возвращаемых бокалов = " + ((n * k - m) / 2);
            LabelHint.Content = hint;

            // ни один бокал не выбран:
            nChose = 0;
            // ход 1:
            nMove = 1;

            // стираем сообщение:
            message = "";
            // печатаем информацию:
            PrintInfo();
            // игра началась:
            gameOver = false;
        }


        // ЩЁЛКАЕМ ПО БОКАЛУ
        private void Bokal_Click(object sender, MouseButtonEventArgs e)
        {
            // игра уже закончилась:
            if (gameOver)
                return;

            // кликнутый бокал:
            var bokal = (Bokal)sender;

            // ранее выбранный бокал 
            // становится невыбранным:
            if (bokal.Chose)
            {
                bokal.Chose = false;
                bokal.Source = imgBlue.Source;
                --nChose;
            }
            // невыбранный бокал
            // становится выбранным:
            else
            {
                bokal.Chose = true;
                bokal.Source = imgRed.Source;
                ++nChose;
            }

            // Игрок выбрал больше бокалов,
            // чем допускают правила:
            if (nChose > nPerev)
            {
                // звук ошибки:
                spError.Play();
            }
            // щелчок по бокалу:
            else
            {
                spChose.Play();
            }

            // сообщение:
            message = "";
            if (nChose > 0)
                message = "Выбрали " + nChose;
            PrintInfo();
        }


        // ПЕЧАТАЕМ ИНФОРМАЦИЮ НА ЭКРАНЕ
        void PrintInfo()
        {
            // печатаем номер хода:
            var str = "Ход " + nMove;
            LabelMove.Content = str;

            // число выбранных или
            // перевёрнутых бокалов:
            LabelChose.Content = message;

            // число неперевёрнутых бокалов:
            str = "";
            if (restBok > 0)
                str = "+" + restBok;
            LabelRestMatch.Content = str;
        }


        // ИГРОК ПЕРЕВОРАЧИВАЕТ БОКАЛЫ
        private void ButtonMove_Click(object sender, RoutedEventArgs e)
        {
            if (gameOver)
                return;

            // игрок выбрал неверное число бокалов:
            if (nChose != nPerev)
            {
                // ошибка:
                spError.Play();
                return;
            }

            // печатаем сообщение:
            message = "Перевернули " + nChose;
            PrintInfo();

            // переворачиваем бокалы:
            Turn();
        }

        // ПЕРЕВОРАЧИВАЕМ ВЫБРАННЫЕ БОКАЛЫ
        async void Turn()
        {
            // список выбранных бокалов:
            var bok = new List<Bokal>();
            // переворачиваем:
            for (int i = 0; i < bokaly.Count; ++i)
            {
                if (bokaly[i].Chose)
                {
                    bok.Add(bokaly[i]);
                    bokaly[i].Chose = false;
                    bokaly[i].RotateBy(180);
                }
            }

            // бокалы становятся синими:
            await Task.Delay(500);
            for (int i = 0; i < bok.Count; ++i)
            {
                bok[i].Up = !bok[i].Up;
                bok[i].Source = imgBlue.Source;
            }

            // осталось неперевёрнутых бокалов:
            restBok = 0;
            foreach (var b in bokaly)
                if (b.Up)
                    ++restBok;
            PrintInfo();
            // выбранных бокалов нет:
            nChose = 0;
            // игра закончилась?
            IsWin();
            // увеличваем счётчик ходов:
            ++nMove;
        }


        // ПРОВЕРЯЕМ, НЕ ЗАКОНЧИЛАСЬ ЛИ ИГРА
        void IsWin()
        {
            if (gameOver)
                return;
            // если не все бокалы перевёрнуты,
            // то игра продолжается:
            if (restBok > 0)
                return;

            spWin.Play();
            // стираем сообщение:
            message = String.Empty;
            PrintInfo();

            imgPozdrBig.Visibility = Visibility.Visible;

            // игра закончена:
            gameOver = true;
        }
    }
}
